@@ -1,5 +1,5 @@
 from websocket import WebSocketApp, create_connection
-from .support import LiveData, TouchlineData
+from .support import TickLiveData, TouchlineData
 
 from threading import Thread
 
@@ -77,11 +77,12 @@ class LiveClient(WebSocketApp):
                 self.parent_app.touchline_data[ticker_id].open = self.parent_app.live_data[ticker_id].day_open = float(contract_details[2])
                 self.parent_app.touchline_data[ticker_id].high = self.parent_app.live_data[ticker_id].day_high = float(contract_details[3])
                 self.parent_app.touchline_data[ticker_id].low = self.parent_app.live_data[ticker_id].day_low = float(contract_details[4])
-                self.parent_app.touchline_data[ticker_id].close = float(contract_details[5])
-                self.parent_app.touchline_data[ticker_id].prev_close = float(contract_details[6])
+                self.parent_app.touchline_data[ticker_id].ltp = float(contract_details[5])
+                self.parent_app.touchline_data[ticker_id].prev_close = self.parent_app.live_data[ticker_id].prev_day_close = float(contract_details[6])
                 self.parent_app.touchline_data[ticker_id].ttq = int(contract_details[7])
                 self.parent_app.touchline_data[ticker_id].oi = int(contract_details[8])
-                self.parent_app.touchline_data[ticker_id].turnover = float(contract_details[9])
+                self.parent_app.touchline_data[ticker_id].prev_oi = self.parent_app.live_data[ticker_id].prev_day_oi = int(contract_details[9])
+                self.parent_app.touchline_data[ticker_id].turnover = float(contract_details[10])
 
     def handle_trade_data(self, trade_tick):
         try:
@@ -89,32 +90,40 @@ class LiveClient(WebSocketApp):
             # print(f'{Style.BRIGHT}{Fore.GREEN} This symbol({symbol}) is tied to req_ids of {self.parent_app.symbol_mkt_id_map[symbol]}...{Style.RESET_ALL}')
             # print(f"{trade_tick} || {type(trade_tick)}")
             for ticker_id in self.parent_app.symbol_mkt_id_map[symbol]:
-                self.parent_app.live_data[ticker_id].timestamp = tick_timestamp = datetime.strptime(trade_tick[1], '%Y-%m-%dT%H:%M:%S')  # Old format = '%m/%d/%Y %I:%M:%S %p'
-                self.parent_app.live_data[ticker_id].symbol = symbol
+                # Assigning new data
                 self.parent_app.live_data[ticker_id].symbol_id = int(trade_tick[0])
-                self.parent_app.live_data[ticker_id].ltp = ltp = float(trade_tick[2])
+                self.parent_app.live_data[ticker_id].timestamp = datetime.strptime(trade_tick[1], '%Y-%m-%dT%H:%M:%S')  # Old format = '%m/%d/%Y %I:%M:%S %p'
+                self.parent_app.live_data[ticker_id].symbol = symbol
+                self.parent_app.live_data[ticker_id].ltp = self.parent_app.touchline_data[ticker_id].ltp = ltp = float(trade_tick[2])
                 self.parent_app.live_data[ticker_id].volume = float(trade_tick[3])
                 self.parent_app.live_data[ticker_id].atp = float(trade_tick[4])
                 self.parent_app.live_data[ticker_id].oi = float(trade_tick[5])
-                self.parent_app.live_data[ticker_id].turnover = float(trade_tick[6])
+                self.parent_app.live_data[ticker_id].ttq = float(trade_tick[6])
+                self.parent_app.live_data[ticker_id].special_tag = special_tag = str(trade_tick[7])
+                if special_tag != "":
+                    if special_tag == 'H':
+                        self.parent_app.live_data[ticker_id].day_high = self.parent_app.touchline_data[ticker_id].high = ltp
+                    elif special_tag == 'L':
+                        self.parent_app.live_data[ticker_id].day_low = self.parent_app.touchline_data[ticker_id].low = ltp
+                self.parent_app.live_data[ticker_id].tick_seq = int(trade_tick[8])
+                # Calculating addn data
+                self.parent_app.live_data[ticker_id].oi_change = self.parent_app.live_data[ticker_id].oi - self.parent_app.live_data[ticker_id].prev_day_oi
+                self.parent_app.live_data[ticker_id].oi_change_perc = self.parent_app.live_data[ticker_id].oi_change * 100 / self.parent_app.live_data[ticker_id].prev_day_oi
+                self.parent_app.live_data[ticker_id].change = self.parent_app.live_data[ticker_id].ltp - self.parent_app.live_data[ticker_id].prev_day_close
+                self.parent_app.live_data[ticker_id].change_perc = self.parent_app.live_data[ticker_id].change * 100 / self.parent_app.live_data[ticker_id].prev_day_close
                 try:
-                    self.parent_app.live_data[ticker_id].best_bid_price = float(trade_tick[7])
-                    self.parent_app.live_data[ticker_id].best_bid_qty = float(trade_tick[8])
-                    self.parent_app.live_data[ticker_id].best_ask_price = float(trade_tick[9])
-                    self.parent_app.live_data[ticker_id].best_ask_qty = float(trade_tick[10])
-                    self.parent_app.live_data[ticker_id].special_tag = special_tag = str(trade_tick[11])
-                    if self.parent_app.live_data[ticker_id].special_tag != "":
-                        if special_tag == 'H':
-                            self.parent_app.live_data[ticker_id].day_high = self.parent_app.touchline_data[ticker_id].high = ltp
-                        elif special_tag == 'L':
-                            self.parent_app.live_data[ticker_id].day_low = self.parent_app.touchline_data[ticker_id].low = ltp
-                except IndexError:
-                    # Happens when not subscribed to bid-ask data in streaming data
-                    pass
-                # ltp = self.parent_app.live_data[ticker_id].ltp
+                    self.parent_app.live_data[ticker_id].best_bid_price = float(trade_tick[9])
+                    self.parent_app.live_data[ticker_id].best_bid_qty = float(trade_tick[10])
+                    self.parent_app.live_data[ticker_id].best_ask_price = float(trade_tick[11])
+                    self.parent_app.live_data[ticker_id].best_ask_qty = float(trade_tick[12])
+                except (IndexError, ValueError, TypeError):
+                    del self.parent_app.live_data[ticker_id].best_bid_price
+                    del self.parent_app.live_data[ticker_id].best_bid_qty
+                    del self.parent_app.live_data[ticker_id].best_ask_price
+                    del self.parent_app.live_data[ticker_id].best_ask_qty
+
         except KeyError:
             print(f'{Style.BRIGHT}{Fore.RED}This symbol is not tied to any req_id...{Style.RESET_ALL}')
-        # print(f"{self.contract_mapping[int(trade_tick[0])]} --> {float(trade_tick[2])} @ {trade_tick[1]}")
 
 
 class HistoricalWebsocket:
@@ -126,7 +135,6 @@ class HistoricalWebsocket:
         self.hist_socket = create_connection(f"wss://{self.url}:{self.historical_port}?user={self.login_id}&password={self.password}")
         welcome_msg = self.hist_socket.recv()
         print(f"Connected successfully to '{json.loads(welcome_msg)['message']}'")
-        self.contract_mapping = {}
 
     def get_hist_bar_data(self, contract, query_time, start_time, bar_size):
         # print(f'{{"method": "gethistory", "interval": "{bar_size}", "symbol": "{contract}", "from": "{start_time}", "to": "{query_time}"}}')
@@ -178,7 +186,21 @@ class HistoricalWebsocket:
                                   'ask_qty': int(j[7])})
             except TypeError:
                 print(f'{Style.BRIGHT}{Fore.RED} {contract} erred with {j}...{Style.RESET_ALL} \n\t {hist_data[count - 2]} \n\t {hist_data[count - 1]} \n\t {hist_data[count]} \n\t {hist_data[count + 1]} \n\t {hist_data[count + 2]}')
+                # If OI is None
+                if j[3] is None:
+                    prev_datapoint_oi = int(hist_data[count-1][3])
+                    data_list.append({'time': datetime.strptime(j[0], '%Y-%m-%dT%H:%M:%S'),
+                                      'ltp': float(j[1]),
+                                      'volume': int(j[2]),
+                                      'oi': prev_datapoint_oi,
+                                      'bid': float(j[4]),
+                                      'bid_qty': int(j[5]),
+                                      'ask': float(j[6]),
+                                      'ask_qty': int(j[7])})
                 continue
+            except IndexError:
+                # Missing bid-ask data
+                pass
             count = count + 1
         return data_list
 
@@ -342,7 +364,7 @@ class TD:
             raise TDLiveDataError
         for j in range(0, len(req_ids)):
             self.touchline_data[req_ids[j]] = TouchlineData()
-            self.live_data[req_ids[j]] = LiveData(resolved_contracts[j])
+            self.live_data[req_ids[j]] = TickLiveData(resolved_contracts[j])
             try:
                 self.symbol_mkt_id_map[resolved_contracts[j].upper()].append(req_ids[j])
             except KeyError:
